@@ -476,11 +476,10 @@ pb_get_fn (BonoboPropertyBag *bag, BonoboArg *arg,
 	time_t val;
 	extern int daylight;
 
-	log = CONFIG_LOG (config_log_open (archiver_db->location));
+	log = location_get_config_log (archiver_db->location);
 	id = config_log_get_rollback_id_by_steps (log, 0, archiver_db->real_name);
 	if (id < 0)
 	{
-		gtk_object_destroy (GTK_OBJECT (log));
 		BONOBO_ARG_SET_GENERAL (arg, 0,
 				TC_ulonglong, CORBA_unsigned_long_long, NULL);
 		return;
@@ -494,7 +493,6 @@ pb_get_fn (BonoboPropertyBag *bag, BonoboArg *arg,
 
 	BONOBO_ARG_SET_GENERAL (arg, val,
 				TC_ulonglong, CORBA_unsigned_long_long, NULL);
-	gtk_object_destroy (GTK_OBJECT (log));
 }
 
 static void
@@ -528,13 +526,7 @@ bonobo_config_archiver_destroy (GtkObject *object)
 
 	if (archiver_db->fp)
 		fclose (archiver_db->fp);
-#if 0
-	if (archiver_db->es)
-		bonobo_object_unref (BONOBO_OBJECT (archiver_db->es));
-#endif
-	if (archiver_db->pb)
-		bonobo_object_unref (BONOBO_OBJECT (archiver_db->pb));
-			
+	
 	parent_class->destroy (object);
 }
 
@@ -639,15 +631,6 @@ fill_cache (BonoboConfigArchiver *archiver_db)
 	}
 }
 
-/* TEMPORARY */
-
-static void
-unref_cb (GtkObject *db, GtkObject *ref_obj) 
-{
-	DEBUG_MSG ("Enter");
-	gtk_object_unref (ref_obj);
-}
-
 Bonobo_ConfigDatabase
 bonobo_config_archiver_new (const char *backend_id, const char *location_id)
 {
@@ -657,7 +640,6 @@ bonobo_config_archiver_new (const char *backend_id, const char *location_id)
 	gchar                 *real_name;
 
 	static Archive        *archive = NULL;
-	static GtkObject      *ref_obj = NULL;
 
 	g_return_val_if_fail (backend_id != NULL, NULL);
 
@@ -756,20 +738,5 @@ bonobo_config_archiver_new (const char *backend_id, const char *location_id)
 
 	bonobo_url_register ("BONOBO_CONF:ARCHIVER", real_name, NULL, db, &ev);
 
-	if (ref_obj == NULL) {
-		ref_obj = gtk_object_new (gtk_object_get_type (), NULL);
-		gtk_signal_connect (ref_obj, "destroy", GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
-		gtk_signal_connect_object (ref_obj, "destroy",
-					   GTK_SIGNAL_FUNC (gtk_object_destroy),
-					   GTK_OBJECT (archive));
-	} else {
-		gtk_object_ref (ref_obj);
-	}
-
-	DEBUG_MSG ("_ref complete");
-
-	gtk_signal_connect (GTK_OBJECT (archiver_db), "destroy",
-			    GTK_SIGNAL_FUNC (unref_cb), ref_obj);
-	
 	return db;
 }
