@@ -34,6 +34,7 @@
 #include "rc-parse.h"
 #include <gal/e-table/e-table.h>
 #include <gal/e-table/e-table-simple.h>
+#include <gal/widgets/e-unicode.h>
 
 #define WID(str) (glade_xml_get_widget (prefs_widget->priv->xml, str))
 
@@ -53,6 +54,7 @@ struct _PrefsWidgetPrivate
 	guint random_timeout;
 	GList *random_current;
 	GtkWindow *parent;
+	gchar **translated;
 
 	/* Copied from Preferences...for OK/Cancel support in dialog */
 	gboolean  power_management;
@@ -90,6 +92,9 @@ static void set_random_timeout            (PrefsWidget *prefs_widget,
 static void popup_item_menu               (ETable *table,
 					   int row, int col, GdkEvent *event,
 					   PrefsWidget *prefs_widget);
+static void double_click_cb		  (ETable *table,
+					   int row, int col, GdkEvent *event,
+					   PrefsWidget *prefs_widget);
 
 static void about_cb                      (GtkWidget *widget,
 					   PrefsWidget *prefs_widget);
@@ -168,8 +173,20 @@ guint prefs_widget_get_type (void)
 static void
 prefs_widget_destroy (PrefsWidget *prefs_widget)
 {
+	int i;
+
 	if (prefs_widget->priv->xml)
 		gtk_object_destroy (GTK_OBJECT (prefs_widget->priv->xml));
+
+	if (prefs_widget->priv->translated)
+	{
+		for (i = 0; prefs_widget->priv->translated[i] != NULL; i++)
+			g_free (prefs_widget->priv->translated[i]);
+		g_free (prefs_widget->priv->translated);
+	}
+
+	close_preview ();
+
 	g_free (prefs_widget->priv);
 
 	if (GTK_OBJECT_CLASS (prefs_widget_parent_class)->destroy)
@@ -237,10 +254,16 @@ prefs_widget_init (PrefsWidget *prefs_widget)
 	gtk_signal_connect (GTK_OBJECT (table), "selection_change",
 			    GTK_SIGNAL_FUNC (selection_changed_cb),
 			    prefs_widget);
+	/* Disabled for now -- no need for Add/Remove/About */
+#if 0
 	gtk_signal_connect (GTK_OBJECT (table), "right_click",
 			    GTK_SIGNAL_FUNC (popup_item_menu),
 			    prefs_widget);
-	
+#endif
+	gtk_signal_connect (GTK_OBJECT (table), "double_click",
+			    GTK_SIGNAL_FUNC (double_click_cb),
+			    prefs_widget);
+
 	widget = WID ("mode_option");
 	option_menu_connect (GTK_OPTION_MENU (widget),
 			     GTK_SIGNAL_FUNC (mode_changed_cb),
@@ -525,7 +548,27 @@ prefs_widget_set_mode (PrefsWidget *prefs_widget, SelectionMode mode)
 void
 prefs_widget_set_screensavers (PrefsWidget *prefs_widget, GList *screensavers)
 {
+	int i;
+	GList *l;
+	Screensaver *saver;
+
+	if (prefs_widget->priv->translated)
+	{
+		for (i = 0; prefs_widget->priv->translated[i] != NULL; i++)
+			g_free (prefs_widget->priv->translated[i]);
+		g_free (prefs_widget->priv->translated);
+	}
+	
 	prefs_widget->screensavers = screensavers;
+	/* Good thing this is called infrequently */
+	prefs_widget->priv->translated = g_new0 (gchar*,
+			g_list_length (screensavers) + 1);
+	for (l = screensavers, i = 0; l != NULL; l = l->next, i++)
+	{
+		saver = l->data;
+		prefs_widget->priv->translated[i] = e_utf8_from_locale_string (gettext (saver->label));
+	}
+	
 	e_table_model_changed (prefs_widget->priv->etm);
 }
 
@@ -574,14 +617,16 @@ model_value_at (ETableModel *etm, int col, int row, void *data)
 {
 	PrefsWidget *prefs_widget = PREFS_WIDGET (data);
 	Screensaver *saver = g_list_nth_data (prefs_widget->screensavers, row);
-
+	
 	if (!saver)
 		return NULL;
 	
 	if (col == 0)
 		return GINT_TO_POINTER (saver->enabled);
 	else
-		return saver->label;
+	{
+		return prefs_widget->priv->translated[row]; 
+	}
 }
 
 static void
@@ -604,7 +649,12 @@ model_set_value_at (ETableModel *etm,
 static gboolean
 model_is_cell_editable (ETableModel *etm, int col, int row, void *data)
 {
-	return (col == 0);
+	PrefsWidget *prefs_widget = PREFS_WIDGET (data);
+
+	if (prefs_widget->selection_mode == SM_CHOOSE_FROM_LIST)
+		return (col == 0);
+	else
+		return FALSE;
 }
 
 static void *
@@ -694,7 +744,7 @@ static void
 selection_foreach_func (int model_row, int *closure)
 {
 	g_return_if_fail (closure != NULL);
-	
+
 	/* Selection mode is "single */
 	*closure = model_row;
 }
@@ -703,7 +753,9 @@ static gint
 random_timeout_cb (PrefsWidget *prefs_widget)
 {
 	GList *l, *old;
- 
+	gboolean skippedwc = FALSE, skippedvw = FALSE;
+	gboolean beenhere = FALSE;
+	
   	g_return_val_if_fail (prefs_widget != NULL, FALSE);
 	
 	l = prefs_widget->priv->random_current;
@@ -716,6 +768,35 @@ random_timeout_cb (PrefsWidget *prefs_widget)
 		/* Handles both "l initially NULL" and "end of list" */
 		if (!l)
 			l = prefs_widget->screensavers;
+		old = l;
+		/* Strip out vidwhacker and webcollage -- don't want to sneak
+		 * those into a random mode if the user didn't select them.
+		 * Especially as a preview. */
+		while (l)
+		{
+			if (skippedwc && skippedvw && old == l)
+			{
+				show_blank_preview ();
+				return FALSE;
+			}
+			if (!strcmp (((Screensaver*)l->data)->name, "webcollage"))
+			{
+				skippedwc = TRUE;
+				l = l->next;
+				if (!l)
+					l = prefs_widget->screensavers;
+				continue;
+			}
+			if (!strcmp (((Screensaver*)l->data)->name, "vidwhacker"))
+			{
+				skippedvw = TRUE;
+				l = l->next;
+				if (!l)
+					l = prefs_widget->screensavers;
+				continue;
+			}
+			break;
+		}
 	}
 	else
 	/* Skip the non-enabled ones */
@@ -730,21 +811,21 @@ random_timeout_cb (PrefsWidget *prefs_widget)
 				l = prefs_widget->screensavers;
 		}
 	
-		if (l)
-			old = l->next;
-		else
-			old = NULL;
+		old = l;
 
 		while (l)
 		{
 			/* Are we back to where we started? */
 			if (((Screensaver*) l->data)->enabled
 			    || l == prefs_widget->priv->random_current
-			    || (old && old == l))
+			    || (beenhere && old && old == l))
 				break;
 			
 			l = l->next;
 			
+			if (!beenhere)
+				beenhere = TRUE;
+			
 			if (!l)
 				l = prefs_widget->screensavers;
 		}
@@ -812,6 +893,14 @@ popup_item_menu (ETable *table,
 			event->button.button, event->button.time);
 }
 
+static void
+double_click_cb (ETable *table,
+		 int row, int col, GdkEvent *event,
+		 PrefsWidget *prefs_widget)
+{
+	settings_cb (NULL, prefs_widget);
+}
+
 static void
 about_cb (GtkWidget *widget, PrefsWidget *prefs_widget)
 {
@@ -820,11 +909,15 @@ about_cb (GtkWidget *widget, PrefsWidget *prefs_widget)
 	gchar *desc, *name;
 
 	desc = screensaver_get_desc (prefs_widget->selected_saver);
-	label = gtk_label_new (desc);
+	label = gtk_label_new (gettext (desc));
 	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
 	
+#if 0
 	name = screensaver_get_label (prefs_widget->selected_saver->name);
-	title = g_strdup_printf ("About %s\n", name);
+#endif
+	name = g_strdup (gettext (prefs_widget->selected_saver->label));
+	
+	title = g_strdup_printf (_("About %s\n"), name);
 	g_free (name);
 	
 	dlg = gnome_dialog_new (title, GNOME_STOCK_BUTTON_CLOSE, NULL);
@@ -843,6 +936,19 @@ about_cb (GtkWidget *widget, PrefsWidget *prefs_widget)
 	gtk_widget_show_all (dlg);
 }
 
+static void
+prefs_ok_cb (ScreensaverPrefsDialog *dlg, PrefsWidget *widget)
+{
+	if (dlg->saver == widget->selected_saver)
+		show_preview (widget->selected_saver);
+}
+
+static void
+prefs_demo_cb (ScreensaverPrefsDialog *dlg, PrefsWidget *widget)
+{
+	show_preview (widget->selected_saver);
+}
+
 static void
 settings_cb (GtkWidget *button, PrefsWidget *widget) 
 {
@@ -851,14 +957,12 @@ settings_cb (GtkWidget *button, PrefsWidget *widget)
 	if (!widget->selected_saver) return;
 
 	dialog = screensaver_prefs_dialog_new (widget->selected_saver);
-#if 0
-	gtk_signal_connect (GTK_OBJECT (dialog), "ok-clicked",
-			    GTK_SIGNAL_FUNC (screensaver_prefs_ok_cb), 
+	gtk_signal_connect_after (GTK_OBJECT (dialog), "ok-clicked",
+			    GTK_SIGNAL_FUNC (prefs_ok_cb), 
 			    widget);
-	gtk_signal_connect (GTK_OBJECT (dialog), "demo",
+	gtk_signal_connect_after (GTK_OBJECT (dialog), "demo",
 			    GTK_SIGNAL_FUNC (prefs_demo_cb), 
 			    widget);
-#endif
 	gtk_widget_show_all (dialog);
 }
 
